{
  pkgs,
  config,
  lib,
  ...
}:
let
  inherit (lib) mkEnableOption mkIf;

  cfg = config.mjm.postgresql;
  secrets = config.vault-secrets.services.postgresql.keys;
in
{
  options.mjm.postgresql = {
    enable = mkEnableOption "postgresql";
  };

  config = mkIf cfg.enable {
    deployment.tags = [ "svc-postgresql" ];
    mjm.state.directories = [
      {
        directory = "/var/lib/postgresql";
        user = "postgres";
        group = "postgres";
        mode = "0750";
      }
    ];

    services.postgresql = {
      enable = true;
      package = pkgs.postgresql_16;
    };

    mjm.backups.postgresql =
      let
        pg = config.services.postgresql.package;
      in
      {
        passwordFile = secrets.backup_password.path;
        paths = [ "/tmp/pgbackup" ];
        user = "postgres";
        backupPrepareCommand = ''
          set -x
          mkdir /tmp/pgbackup
          cd /tmp/pgbackup
          ${pg}/bin/pg_dumpall --globals-only -f /tmp/pgbackup/globals.sql
          ${lib.concatMapStrings (dbname: ''
            ${pg}/bin/pg_dump --format=directory -j 4 -f ${dbname} ${dbname}
          '') config.services.postgresql.ensureDatabases}
        '';
        backupCleanupCommand = ''
          rm -rf /tmp/pgbackup
        '';
      };

    vault.services.postgresql = { };
    vault-secrets.services.postgresql = {
      keys.backup_password = {
        owner = "postgres";
      };
    };

    deployment.tests = {
      inherit (pkgs.nixosTests.postgresql.postgresql.postgresql_16)
        postgresql
        postgresql-backup-all
        ;
    };
  };
}
